"_PACKAGE"

#' rhdf5: An interface between HDF5 and R
#'
#' The rhdf5 package provides two categories of functions:
#' * `h5` functions are high-level R functions that provide a convenient way of
#'   accessing HDF5 files
#' * `H5` functions mirror much of the the HDF5 C API
#'
#' @name rhdf5
#'
#' @import Rhdf5lib
#' @import rhdf5filters
#' @import methods
#'
#' @useDynLib rhdf5
NULL
