% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Deprecated.R
\name{H5Pget_version}
\alias{H5Pget_version}
\title{Deprecated}
\usage{
H5Pget_version(...)
}
\arguments{
\item{...}{ignored}
}
\value{
NULL (invisibly)
}
\description{
Deprecated in line with changes in the upstream HDF5 library.
It will be removed in the next version of rhdf5.
}
