\name{h5delete}
\alias{h5delete}

\title{Delete contents of HDF5 file}

\description{Deletes the specfied group or dataset from within an HDF5 file.}

\usage{
    h5delete( file, name )
}

\arguments{
  \item{file}{The filename (character) of the file in which the object is located.}
  \item{name}{The names of the object to be deleted.}
}

\author{Mike Smith}

\keyword{ file }
