% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5D_extras.R
\name{H5Dchunk_dims}
\alias{H5Dchunk_dims}
\title{Return the dimensions of a dataset chunk}
\usage{
H5Dchunk_dims(h5dataset)
}
\arguments{
\item{h5dataset}{Object of class \linkS4class{H5IdComponent} representing an open HDF5
dataset.}
}
\value{
If the supplied dataset is chunked returns a vector, with length
equal to the rank of the dataset, containing the size of the dataset
dimensions.  Returns \code{NULL} if the given dataset is not chunked.
}
\description{
Return the dimensions of a dataset chunk
}
\details{
This function does not map directly to the HDF5 C API but is
included as a useful addition.
}
\author{
Mike Smith
}
