% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5constants.R
\name{h5constants}
\alias{h5constants}
\alias{h5const}
\alias{h5constType}
\alias{h5default}
\title{HDF5 library constants.}
\usage{
h5const(type = "")

h5constType()

h5default(type = "")
}
\arguments{
\item{type}{A character name of a group of constants.}
}
\value{
A character vector with names of HDF5 constants or groups.
}
\description{
Access to HDF5 constants.
}
\details{
These functions provide a list of HDF5 constants that are defined in the R
package. \code{h5constType} provides a list of group names and
\code{h5const} gives the constants defined within a group. \code{h5default}
gives the default choice for each group.
}
\examples{

h5constType()[1]
h5const(h5constType()[1])
}
\author{
Bernd Fischer
}
