% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5D.R
\name{H5Dwrite}
\alias{H5Dwrite}
\title{Write data to dataset}
\usage{
H5Dwrite(h5dataset, buf, h5spaceMem = NULL, h5spaceFile = NULL)
}
\arguments{
\item{h5dataset}{Object of class \linkS4class{H5IdComponent} representing an open HDF5
dataset.}

\item{buf}{The R object containing the data to be written to the dataset.}

\item{h5spaceMem, h5spaceFile}{\linkS4class{H5IdComponent} objects representing the
memory and file dataspaces respectively.  If these are left \code{NULL} dataspaces
that match the size and shape of \code{h5dataset} will be used.}
}
\description{
Write data to dataset
}
