% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5D.R
\name{H5Dget_storage_size}
\alias{H5Dget_storage_size}
\title{Find the amount of storage allocated for a dataset}
\usage{
H5Dget_storage_size(h5dataset)
}
\arguments{
\item{h5dataset}{Object of class \linkS4class{H5IdComponent} representing an open HDF5
dataset}
}
\value{
Returns an integer giving the number of bytes allocated in the file
to the dataset.
}
\description{
\code{H5Dget_storage_size} returns the amount of storage, in bytes, allocated in
an HDF5 file to hold a given dataset.  This is the amount of space required
on-disk, which not typically a good indicator of the amount of memory
that will be required to read the complete dataset.
}
