% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5G.R
\name{H5Gcreate}
\alias{H5Gcreate}
\title{Create a new HDF5 group and link it to a location in a file}
\usage{
H5Gcreate(h5loc, name)
}
\arguments{
\item{h5loc}{An object of class \linkS4class{H5IdComponent}}

\item{name}{Name of the new group to be created.}
}
\description{
\code{H5Gcreate} is used to a new group and link it into a file.
}
