% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5P.R
\name{H5Pset_filter}
\alias{H5Pset_filter}
\title{Add a filter to the dataset filter pipeline.}
\usage{
H5Pset_filter(h5plist, filter_id, is_mandatory = FALSE, cd_values)
}
\arguments{
\item{h5plist}{Object of class \linkS4class{H5IdComponent} representing a dataset
creation property list.}

\item{filter_id}{Integer of length 1, giving the ID of the filter to be used.}

\item{is_mandatory}{Logical of length 1.  Filters can be either optional or
mandatory.  If this argument is set to \code{FALSE} the filter won't be applied
to a chunk in the case of failure, but the data will still be written.
Setting to \code{TRUE} will result in a failure when writing the dataset if the
filter fails for some reason.}

\item{cd_values}{Integer vector giving parameters to be supplied to the
filter. No guidance is given for the number of values supplied here, it is
specific to each filter and the user is expected to know appropriate
options for the requested filter.}
}
\description{
Add a filter to the dataset filter pipeline.
}
