% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/h5delete.R
\name{h5_delete}
\alias{h5_delete}
\alias{h5delete}
\title{Delete objects within a HDF5 file}
\usage{
h5delete(file, name)
}
\arguments{
\item{file}{The filename (character) of the file in which the object is
located.}

\item{name}{For \code{h5delete} the name of the object to be deleted. For
\code{h5deleteAttribute} the name of the object to which the attribute
belongs.}
}
\description{
Deletes the specified group or dataset from within an HDF5 file.
}
\author{
Mike Smith
}
