% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5O.R
\name{H5Oclose}
\alias{H5Oclose}
\title{Close an HDF5 object}
\usage{
H5Oclose(h5obj)
}
\arguments{
\item{h5obj}{An object of class \linkS4class{H5IdComponent} representing an open
HDF5 object.}
}
\description{
Close an HDF5 object
}
\seealso{
\code{\link[=H5Oopen]{H5Oopen()}}
}
