% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5P.R
\name{H5Pcreate}
\alias{H5Pcreate}
\title{Create a new HDF5 property list}
\usage{
H5Pcreate(type = h5default("H5P"), native = FALSE)
}
\arguments{
\item{type}{A character name of a property list type. See \code{h5const("H5P")}
for possible property list types.}

\item{native}{Defunct! Doesn't achieve anything for property lists.}
}
\description{
Create a new HDF5 property list
}
