% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5S.R
\name{H5Sclose}
\alias{H5Sclose}
\title{Close and release a dataspace}
\usage{
H5Sclose(h5space)
}
\arguments{
\item{h5space}{Object of class \linkS4class{H5IdComponent} representing the
dataspace to be closed.}
}
\description{
Close and release a dataspace
}
\seealso{
\code{\link[=H5Screate]{H5Screate()}}
}
