% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5L.R
\name{H5Lget_info}
\alias{H5Lget_info}
\title{Find information about a link}
\usage{
H5Lget_info(h5loc, name)
}
\arguments{
\item{h5loc}{An object of class \linkS4class{H5IdComponent} representing a H5
location identifier (file or group).}

\item{name}{The name of the link to be queried.}
}
\value{
A character vector of length 1 giving the type of link.  Possible
values are:
\code{H5L_TYPE_HARD}, \code{H5L_TYPE_SOFT}, \code{H5L_TYPE_EXTERNAL}, \code{H5L_TYPE_ERROR}
}
\description{
\code{H5Lget_info()} identifies the type of link specified by the the \code{h5loc}
and \code{name} arguments. This is more limited than the equivalent function in
the standard HDF5 library.
}
