% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5P.R
\name{H5Pclose}
\alias{H5Pclose}
\title{Close and release a property list}
\usage{
H5Pclose(h5plist)
}
\arguments{
\item{h5plist}{\linkS4class{H5IdComponent} object representing the property list
to close.}
}
\description{
\code{H5Pclose()} terminates access to a property list. All property lists
should be closed when they no longer need to be accessed. This
frees resources used by the property list.  Failing to call \code{H5Pclose()}
can lead to memory leakage over time.
}
