% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5P.R
\name{H5Pcopy}
\alias{H5Pcopy}
\title{Copy an existing property list to create a new property list}
\usage{
H5Pcopy(h5plist)
}
\arguments{
\item{h5plist}{\linkS4class{H5IdComponent} object representing the property list
to be copied.}
}
\description{
Copy an existing property list to create a new property list
}
