% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5S.R
\name{H5Sset_extent_simple}
\alias{H5Sset_extent_simple}
\title{Set the size of a dataspace}
\usage{
H5Sset_extent_simple(h5space, dims, maxdims)
}
\arguments{
\item{h5space}{\linkS4class{H5IdComponent} object representing a dataspace.}

\item{dims}{Dimension of the dataspace. This argument is similar to the dim
attribute of an array. When viewing the HDF5 dataset with an C-program
(e.g. HDFView), the dimensions appear in inverted order, because the
fastest changing dimension in R is the first one, and in C its the last
one.}

\item{maxdims}{Maximum extension of the dimension of the dataset in the
file. If not provided, it is set to \code{dims}.}
}
\description{
Set the size of a dataspace
}
