% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5O_extras.R
\name{H5Oget_num_attrs}
\alias{H5Oget_num_attrs}
\alias{H5Oget_num_attrs_by_name}
\title{Find the number of attributes associated with an HDF5 object}
\usage{
H5Oget_num_attrs(h5obj)

H5Oget_num_attrs_by_name(h5loc, name)
}
\arguments{
\item{h5obj}{An object of class \linkS4class{H5IdComponent} representing a H5 object identifier (file, group, or dataset).}

\item{h5loc}{An object of class \linkS4class{H5IdComponent} representing a H5 location identifier (file or group).}

\item{name}{The name of the object to be checked.}
}
\value{
Returns a vector of length 1 containing the number of attributes
the specified object has.
}
\description{
Find the number of attributes associated with an HDF5 object
}
\details{
These functions are not part of the standard HDF5 C API.
}
