\name{[-methods}
\docType{methods}
\alias{[,H5IdComponent-method}
\title{Methods for Function \code{[} in Package \pkg{rhdf5}}
\description{
Methods for function \code{[} in package \pkg{rhdf5}.
}
\section{Methods}{
\describe{

\item{\code{signature(x = "H5IdComponent")}}{
 Subsetting of an HDF5 dataset. The function reads a subset of an HDF5 dataset. The given dimensions have to fit the dimensions of the HDF5 dataset.
}
}}
\keyword{methods}
\keyword{ programming }
\keyword{ interface }
\keyword{ IO }
\keyword{ file }
