% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5S.R
\name{H5Sunlimited}
\alias{H5Sunlimited}
\title{Retrieve value for \code{H5S_UNLIMITED} constant}
\usage{
H5Sunlimited()
}
\description{
The value for \code{H5S_UNLIMITED} can be provided to the \code{maxdims} argument of \link{H5Screate_simple}
to indicate that the maximum size of the corresponding dimension is unlimited.
}
\seealso{
\link{H5Screate_simple}
}
