% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5P_extras.R
\name{H5Pset_lzf}
\alias{H5Pset_lzf}
\title{Add the LZF filter to the chunk processing pipeline.}
\usage{
H5Pset_lzf(h5plist, h5tid)
}
\arguments{
\item{h5plist}{Object of class \linkS4class{H5IdComponent} representing a dataset
creation property list.}

\item{h5tid}{HDF5 data type id}
}
\description{
Add the LZF filter to the chunk processing pipeline.
}
