% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5S.R
\name{H5Sselect_valid}
\alias{H5Sselect_valid}
\title{Check that a selection is valid}
\usage{
H5Sselect_valid(h5space)
}
\arguments{
\item{h5space}{\linkS4class{H5IdComponent} object representing a dataspace.}
}
\description{
Check that a selection is valid
}
