% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5P.R
\name{H5Pset_sizes}
\alias{H5Pset_sizes}
\alias{H5Pget_sizes}
\title{Get and set the sizes of offsets and lengths used in an HDF5 file}
\usage{
H5Pset_sizes(h5plist, sizeof_addr, sizeof_size)

H5Pget_sizes(h5plist)
}
\arguments{
\item{h5plist}{\linkS4class{H5IdComponent} object representing the file creation property list}

\item{sizeof_addr}{Offset size in bytes}

\item{sizeof_size}{Length size in bytes}
}
\description{
Get and set the sizes of offsets and lengths used in an HDF5 file
}
