% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5A.R
\name{H5Aexists}
\alias{H5Aexists}
\title{Check whether an specific attribute exists for an HDF5 object}
\usage{
H5Aexists(h5obj, name)
}
\arguments{
\item{h5obj}{An object of class \linkS4class{H5IdComponent} representing a H5 object
identifier (file, group, or dataset). See \code{\link[=H5Fcreate]{H5Fcreate()}}, \code{\link[=H5Fopen]{H5Fopen()}},
\code{\link[=H5Gcreate]{H5Gcreate()}}, \code{\link[=H5Gopen]{H5Gopen()}}, \code{\link[=H5Dcreate]{H5Dcreate()}}, or \code{\link[=H5Dopen]{H5Dopen()}} to create an object of this kind.}

\item{name}{The name of the attribute (character).}
}
\description{
Check whether an specific attribute exists for an HDF5 object
}
