% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5D.R
\name{H5Dget_space}
\alias{H5Dget_space}
\title{Return a copy of the HDF5 dataspace for a dataset}
\usage{
H5Dget_space(h5dataset)
}
\arguments{
\item{h5dataset}{Object of class \linkS4class{H5IdComponent} representing an open HDF5
dataset}
}
\value{
Returns an object of class \code{H5IdComponent} representing a HDF5
dataspace identifier
}
\description{
Return a copy of the HDF5 dataspace for a dataset
}
