% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5G.R
\name{H5Gclose}
\alias{H5Gclose}
\title{Close a specified group}
\usage{
H5Gclose(h5group)
}
\arguments{
\item{h5group}{An object of class \linkS4class{H5IdComponent} representing a H5
group.  Typically created via \code{\link[=H5Gopen]{H5Gopen()}} or \code{\link[=H5Gcreate]{H5Gcreate()}}.}
}
\description{
Close a specified group
}
