% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5F.R
\name{H5Fclose}
\alias{H5Fclose}
\title{Close access to an HDF5 file}
\usage{
H5Fclose(h5file)
}
\arguments{
\item{h5file}{\linkS4class{H5IdComponent} representing an HDF5 file ID.  Typically
created via \code{\link[=H5Fcreate]{H5Fcreate()}} or \code{\link[=H5Fopen]{H5Fopen()}}.}
}
\description{
Close access to an HDF5 file
}
