% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rhdf5.R
\name{h5version}
\alias{h5version}
\title{Print the rhdf5 and libhdf5 version numbers}
\usage{
h5version()
}
\value{
A list of major, minor and release number.
}
\description{
Returns the version number of the Bioconductor package rhdf5 and the
C-library libhdf5.
}
\examples{

h5version()

}
\author{
Bernd Fischer, Mike L. Smith
}
