% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5G.R
\name{H5Gcreate_anon}
\alias{H5Gcreate_anon}
\title{Create a new HDF5 group without linking it into a file}
\usage{
H5Gcreate_anon(h5loc)
}
\arguments{
\item{h5loc}{An object of class \linkS4class{H5IdComponent} specifying the file in
which the new group is to be created.}
}
\value{
\code{H5Gcreate_anon} returns an object of class \linkS4class{H5IdComponent}
representing the newly created group.  However at this point is is still
anonymous, and must be linked into the file structure via \code{\link[=H5Olink]{H5Olink()}}.
If this is not done, the group will be deleted from the file when it
is closed.
}
\description{
Create a new HDF5 group without linking it into a file
}
\seealso{
\code{\link[=H5Gcreate]{H5Gcreate()}}, \code{\link[=H5Olink]{H5Olink()}}
}
