% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5L.R
\name{H5Lcopy}
\alias{H5Lcopy}
\title{Copy a link from one location to another}
\usage{
H5Lcopy(h5loc, name, h5loc_dest, name_dest, lcpl = NULL, lapl = NULL)
}
\arguments{
\item{h5loc}{An object of class \linkS4class{H5IdComponent} representing a H5
location identifier (file or group) where the new link is placed.}

\item{name}{The name of the link to be copied.}

\item{h5loc_dest}{An object of class \linkS4class{H5IdComponent} representing
the destination file or group where a copied or moved link should be created.}

\item{name_dest}{The name of the link to be created when copying or moving.}

\item{lcpl, lapl}{Link creation and link access property lists.  If left as
\code{NULL} the HDF5 defaults will be used.}
}
\description{
Copy a link from one location to another
}
