% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5P.R
\name{H5Pset_deflate}
\alias{H5Pset_deflate}
\title{Add the deflate compression filter to the chunk processing pipeline.}
\usage{
H5Pset_deflate(h5plist, level)
}
\arguments{
\item{h5plist}{Object of class \linkS4class{H5IdComponent} representing a dataset
creation property list.}

\item{level}{Integer giving the compression level to use.  Valid values are
from 0 to 9.}
}
\description{
Valid values for the compression level range from 0 (no compression) to 9
(best compression, slowest speed). Note that applying this function with
\code{level = 0} does not mean the filter is removed.  It is still part of the
filter pipeline, but no compression is performed.  The filter will still need
to be available on any system that reads a file created with this setting
}
