% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5P_extras.R
\name{H5Pset_bzip2}
\alias{H5Pset_bzip2}
\title{Add the BZIP2 filter to the chunk processing pipeline.}
\usage{
H5Pset_bzip2(h5plist, level = 2L)
}
\arguments{
\item{h5plist}{Object of class \linkS4class{H5IdComponent} representing a dataset
creation property list.}

\item{level}{Compression level to be used by the selected algorithm.}
}
\description{
Add the BZIP2 filter to the chunk processing pipeline.
}
