% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5P.R
\name{H5Pset_szip}
\alias{H5Pset_szip}
\title{Add the SZIP compression filter to the chunk processing pipeline.}
\usage{
H5Pset_szip(h5plist, options_mask, pixels_per_block)
}
\arguments{
\item{h5plist}{Object of class \linkS4class{H5IdComponent} representing a dataset
creation property list.}

\item{options_mask, pixels_per_block}{Integer vectors of length 1, setting parameters
of the SZIP algorithm. See \url{https://portal.hdfgroup.org/display/HDF5/H5P_SET_SZIP} for more details.}
}
\description{
Add the SZIP compression filter to the chunk processing pipeline.
}
\references{
\url{https://portal.hdfgroup.org/display/HDF5/Szip+Compression+in+HDF+Products}
}
