% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5T.R
\name{H5Tis_variable_str}
\alias{H5Tis_variable_str}
\title{Determine whether a datatype is a variable length string}
\usage{
H5Tis_variable_str(dtype_id)
}
\arguments{
\item{dtype_id}{ID of HDF5 datatype to query.}
}
\description{
Determine whether a datatype is a variable length string
}
