% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5P.R
\name{H5Pset_shared_mesg_nindexes}
\alias{H5Pset_shared_mesg_nindexes}
\alias{H5Pget_shared_mesg_nindexes}
\title{Get and set the number of object header message indexes}
\usage{
H5Pset_shared_mesg_nindexes(h5plist, nindexes)

H5Pget_shared_mesg_nindexes(h5plist)
}
\arguments{
\item{h5plist}{\linkS4class{H5IdComponent} object representing the file creation property list}

\item{nindexes}{Number of shared object header message indexes to be available in files}
}
\description{
Get and set the number of object header message indexes
}
