% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5F.R
\name{H5Fget_plist}
\alias{H5Fget_plist}
\alias{H5Fget_create_plist}
\alias{H5Fget_access_plist}
\title{Get property lists associated with an HDF5 file}
\usage{
H5Fget_create_plist(h5file)

H5Fget_access_plist(h5file)
}
\arguments{
\item{h5file}{An object of class \linkS4class{H5IdComponent} representing a H5
file identifier.  Typically produced by \code{\link[=H5Fopen]{H5Fopen()}} or \code{\link[=H5Fcreate]{H5Fcreate()}}.}
}
\description{
Get property lists associated with an HDF5 file
}
