% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5I.R
\name{H5Iget_name}
\alias{H5Iget_name}
\title{Retrieve the name of an object from a given identifier}
\usage{
H5Iget_name(h5obj)
}
\arguments{
\item{h5obj}{An object of class \linkS4class{H5IdComponent}.  Can represent a
file, group, dataset or attribute.}
}
\description{
Retrieve the name of an object from a given identifier
}
