% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5A.R
\name{H5Aget_name}
\alias{H5Aget_name}
\title{Get the name of an HDF5 attribute object}
\usage{
H5Aget_name(h5attribute)
}
\arguments{
\item{h5attribute}{An object of class \linkS4class{H5IdComponent} representing an
attribute.  Normally created by \code{\link[=H5Aopen]{H5Aopen()}} or similar.}
}
\value{
A character vector of length 1 containing the name of the attribute.
}
\description{
Retrieves the name of the attribute specified by an HDF5 attribute object.
}
