% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5R.R
\name{H5Rget_name}
\alias{H5Rget_name}
\title{Return the name of the object that a reference points to}
\usage{
H5Rget_name(ref, h5loc)
}
\arguments{
\item{ref}{\code{H5ref} object containing the reference to be queried.}

\item{h5loc}{An \code{H5IdComponent} object representing the file containing the
referenced object.}
}
\value{
Character string of length 1 giving the name of the referenced
object.
}
\description{
Return the name of the object that a reference points to
}
