% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5P.R
\name{H5Pget_class}
\alias{H5Pget_class}
\title{Return the property list class identifier for a property list}
\usage{
H5Pget_class(h5plist)
}
\arguments{
\item{h5plist}{\linkS4class{H5IdComponent} object representing any type of HDF5
property list.}
}
\description{
Return the property list class identifier for a property list
}
