% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5P.R
\name{H5P_chunk_cache}
\alias{H5P_chunk_cache}
\alias{H5Pset_chunk_cache}
\title{Set parameters for the raw data chunk cache}
\usage{
H5Pset_chunk_cache(h5plist, rdcc_nslots, rdcc_nbytes, rdcc_w0)
}
\arguments{
\item{h5plist}{Object of class \linkS4class{H5IdComponent} representing a dataset
access property list.}

\item{rdcc_nslots}{Integer defining the number of chunk slots in the raw data
chunk cache for this dataset.}

\item{rdcc_nbytes}{Integer setting the total size of the raw data chunk cache
for this dataset in bytes. In most cases increasing this number will
improve performance, as long as you have enough free memory. The default
size is 1 MB}

\item{rdcc_w0}{Numeric value defining the chunk preemption policy.  Must be between
\code{0} and \code{1} inclusive.}
}
\description{
Set parameters for the raw data chunk cache
}
