% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5A.R
\name{H5Aget_space}
\alias{H5Aget_space}
\title{Get a copy of the attribute dataspace}
\usage{
H5Aget_space(h5attribute)
}
\arguments{
\item{h5attribute}{An object of class \linkS4class{H5IdComponent} representing an
attribute.  Normally created by \code{\link[=H5Aopen]{H5Aopen()}} or similar.}
}
\value{
Returns an object of class \linkS4class{H5IdComponent} representing a H5
dataspace identifier
}
\description{
Get a copy of the attribute dataspace
}
