% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5P.R
\name{H5Pset_sym_k}
\alias{H5Pset_sym_k}
\alias{H5Pget_sym_k}
\title{Get and set the size of the symbol table B-tree 1/2 rank and the leaf node 1/2 size}
\usage{
H5Pset_sym_k(h5plist, ik, lk)

H5Pget_sym_k(h5plist)
}
\arguments{
\item{h5plist}{\linkS4class{H5IdComponent} object representing the file creation property list}

\item{ik}{Symbol table B-tree 1/2 rank}

\item{lk}{Symbol table leaf node 1/2 size}
}
\description{
Get and set the size of the symbol table B-tree 1/2 rank and the leaf node 1/2 size
}
