% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5A.R
\name{H5Awrite}
\alias{H5Awrite}
\title{Write data to an HDF5 attribute}
\usage{
H5Awrite(h5attribute, buf)
}
\arguments{
\item{h5attribute}{An object of class \linkS4class{H5IdComponent} representing an
attribute.  Normally created by \code{\link[=H5Aopen]{H5Aopen()}} or similar.}

\item{buf}{The data to be written.}
}
\description{
Write data to an HDF5 attribute
}
