% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5T.R
\name{H5T_strpad}
\alias{H5T_strpad}
\alias{H5Tset_strpad}
\alias{H5Tget_strpad}
\title{Retrieve or set the type of padding used by string datatype}
\usage{
H5Tset_strpad(dtype_id, strpad = "NULLPAD")

H5Tget_strpad(dtype_id)
}
\arguments{
\item{dtype_id}{ID of HDF5 datatype to query or modify.}

\item{strpad}{Character vector of length 1 specifying the type of padding
to use.  Valid options are \code{NULLTERM}, \code{NULLPAD} and \code{SPACEPAD}.}
}
\description{
Retrieve or set the type of padding used by string datatype
}
