% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5T.R
\name{H5T_cset}
\alias{H5T_cset}
\alias{H5Tset_cset}
\alias{H5Tget_cset}
\title{Retrieve or set the character set to be used in a string datatype.}
\usage{
H5Tset_cset(dtype_id, cset = "ASCII")

H5Tget_cset(dtype_id)
}
\arguments{
\item{dtype_id}{ID of HDF5 datatype to query or modify.}

\item{cset}{Encoding to use for string types. Valid options are 'ASCII' and
'UTF-8'.}
}
\description{
Retrieve or set the character set to be used in a string datatype.
}
