% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5P.R
\name{H5Pset_userblock}
\alias{H5Pset_userblock}
\alias{H5Pget_userblock}
\title{Get and set the user block size}
\usage{
H5Pset_userblock(h5plist, size)

H5Pget_userblock(h5plist)
}
\arguments{
\item{h5plist}{\linkS4class{H5IdComponent} object representing the file creation property list}

\item{size}{of the user block in bytes}
}
\description{
Get and set the user block size
}
