% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5D.R
\name{H5Dclose}
\alias{H5Dclose}
\title{Close an open HDF5 dataset}
\usage{
H5Dclose(h5dataset)
}
\arguments{
\item{h5dataset}{Object of class \linkS4class{H5IdComponent} representing an open HDF5
dataset}
}
\description{
Close an open HDF5 dataset
}
