% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5T.R
\name{H5T_precision}
\alias{H5T_precision}
\alias{H5Tset_precision}
\alias{H5Tget_precision}
\title{Retrieve or set the precision of an HDF5 datatype}
\usage{
H5Tset_precision(dtype_id, precision)

H5Tget_precision(dtype_id)
}
\arguments{
\item{dtype_id}{ID of HDF5 datatype to set precision of.}

\item{precision}{The number of bytes of precision for the datatype.}
}
\value{
\itemize{
\item \code{H5Tget_precision()} returns an integer giving the number of
significant bits used by the given datatype.
\item \code{H5Tset_precision()} is call for its side-effect of modifying the
precision of a datatype.  It will invisibly return \code{TRUE} if this is
successful and will stop with an error if the operation fails.
}
}
\description{
Retrieve or set the precision of an HDF5 datatype
}
