% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5P_extras.R
\name{H5Pset_blosc}
\alias{H5Pset_blosc}
\title{Add the BLOSC filter to the chunk processing pipeline.}
\usage{
H5Pset_blosc(h5plist, h5tid, method = 1L, level = 6L, shuffle = TRUE)
}
\arguments{
\item{h5plist}{Object of class \linkS4class{H5IdComponent} representing a dataset
creation property list.}

\item{h5tid}{HDF5 data type id}

\item{method}{Integer defining which of the compression algorithms provided
by BLOSC should be used. (See the details section for the mapping between
integers and algorithms).}

\item{level}{Compression level to be used by the selected algorithm.}

\item{shuffle}{Logical defining whether the bit-shuffle algorithm should
be used prior to compression.  This makes use of the shuffle implementation
provide by BLOSC, rather than the HDF5 version.}
}
\description{
Add the BLOSC filter to the chunk processing pipeline.
}
