% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Rhdf5.R
\docType{package}
\name{rhdf5}
\alias{rhdf5}
\title{rhdf5: An interface between HDF5 and R}
\description{
The rhdf5 package provides two categories of functions:
\itemize{
\item \code{h5} functions are high-level R functions that provide a convenient way of accessing HDF5 files
\item \code{H5} functions mirror much of the the HDF5 C API
}
}
