% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5T.R
\name{H5Tcopy}
\alias{H5Tcopy}
\title{Copy an existing datatype}
\usage{
H5Tcopy(dtype_id = h5default(type = "H5T"))
}
\arguments{
\item{dtype_id}{Datatype to copy.  Can either be a character
specifying a predefined HDF5 datatype (see \code{h5const("H5T")} for valid
options) or the ID of an already created datatype.}
}
\description{
Copy an existing datatype
}
