% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/h5errorHandling.R
\name{h5_errorHandling}
\alias{h5_errorHandling}
\alias{h5errorHandling}
\title{Set how HDF5 error messages are displayed}
\usage{
h5errorHandling(type = "normal")
}
\arguments{
\item{type}{'normal' (default) shows a one line error message in R.
'verbose' shows the whole HDF5 error message. 'suppress' suppresses the HDF5
error messages completely.}
}
\value{
Returns 0 if options are set successfully.
}
\description{
Sets the options for handling HDF5 error messages in the R sessions.
}
\examples{

h5errorHandling("normal")

}
\seealso{
\link{rhdf5}
}
\author{
Bernd Fischer
}
