% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5P.R
\name{H5P_fill_value}
\alias{H5P_fill_value}
\alias{H5Pset_fill_value}
\title{Set the fill value for an HDF5 dataset}
\usage{
H5Pset_fill_value(h5plist, value)
}
\arguments{
\item{h5plist}{An object of class \linkS4class{H5IdComponent} representing a
dataset creation property list.}

\item{value}{The default fill value of the dataset. A vector of length 1.}
}
\description{
\code{H5Pset_fill_value} sets the fill value for a dataset in the dataset creation property list.
}
\seealso{
\link{H5P_fill_time},\link{H5Pfill_value_defined}
}
