% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5D.R
\name{H5Dset_extent}
\alias{H5Dset_extent}
\title{Change the dimensions of an HDF5 dataset}
\usage{
H5Dset_extent(h5dataset, size)
}
\arguments{
\item{h5dataset}{Object of class \linkS4class{H5IdComponent} representing an open HDF5
dataset.}

\item{size}{An integer vector with the new dimension of the dataset.}
}
\description{
Change the dimensions of an HDF5 dataset
}
\details{
This function can only be applied to datasets that meet the
following criteria:
\itemize{
\item A chunked dataset with unlimited dimensions
\item A chunked dataset with fixed dimensions if the new dimension sizes are
less than the maximum sizes set with maxdims
#'
}
}
\author{
Bernd Fischer, Mike Smith
}
