% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5S.R
\name{H5Scopy}
\alias{H5Scopy}
\title{Create a copy of a dataspace}
\usage{
H5Scopy(h5space)
}
\arguments{
\item{h5space}{Object of class \linkS4class{H5IdComponent} representing the
dataspace to be copied.}
}
\value{
If the copying is successful returns an object of class
\linkS4class{H5IdComponent} representing the new dataspace.  Otherwise returns
\code{FALSE}.
}
\description{
\code{H5S_copy()} creates an exact copy of a given dataspace.
}
