% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5Z.R
\name{H5Zfilter_avail}
\alias{H5Zfilter_avail}
\title{Determine whether a filter is available on this system}
\usage{
H5Zfilter_avail(filter_id)
}
\arguments{
\item{filter_id}{Integer representing the ID of the filter to be checked.}
}
\description{
Determine whether a filter is available on this system
}
