% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5.R
\name{H5functions}
\alias{H5functions}
\alias{H5open}
\alias{H5close}
\alias{H5garbage_collect}
\alias{H5get_libversion}
\title{HDF5 General Library Functions}
\usage{
H5open()

H5close()

H5garbage_collect()

H5get_libversion()
}
\value{
\itemize{
\item \code{H5open} initializes the HDF5 library.
\item \code{H5close} flushes all data to disk, closes all open identifiers, and cleans up memory.
\item \code{H5garbage_collect} cleans up memory.
\item \code{H5get_libversion} returns the version number of the HDF5 C-library.
}
}
\description{
These low level functions provide general library functions for HDF5.
}
\examples{
\dontrun{
  H5open()
  H5close()
  H5garbage_collect()
  H5get_libversion()
}
}
\author{
Bernd Fischer, Mike Smith
}
