% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5P.R
\name{H5Pfill_value_defined}
\alias{H5Pfill_value_defined}
\title{Determine whether a property list has a fill value defined}
\usage{
H5Pfill_value_defined(h5plist)
}
\arguments{
\item{h5plist}{Object of class \linkS4class{H5IdComponent} representing a dataset
creation property list.}
}
\value{
\code{TRUE} if the fill value is defined, \code{FALSE} if not.  Will return
\code{NULL} if there is a problem determining the status of the fill value.
}
\description{
Determine whether a property list has a fill value defined
}
\details{
Note that the return value for this function is slightly different
from the C version.  The C API provides three return types and can, in the
case that a fill value is defined, differentiate whether the value is the
HDF5 library default or has been set by the application.
}
