% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5S.R
\name{H5Sget_select_npoints}
\alias{H5Sget_select_npoints}
\title{Find the number of elements in a dataspace selection}
\usage{
H5Sget_select_npoints(h5space)
}
\arguments{
\item{h5space}{\linkS4class{H5IdComponent} object representing a dataspace.}
}
\description{
Find the number of elements in a dataspace selection
}
