% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/h5dump.R
\name{h5_dump}
\alias{h5_dump}
\alias{h5dump}
\title{Dump the content of an HDF5 file.}
\usage{
h5dump(
  file,
  recursive = TRUE,
  load = TRUE,
  all = FALSE,
  index_type = h5default("H5_INDEX"),
  order = h5default("H5_ITER"),
  s3 = FALSE,
  s3credentials = NULL,
  ...,
  native = FALSE
)
}
\arguments{
\item{file}{The filename (character) of the file in which the dataset will
be located. You can also provide an object of class \linkS4class{H5IdComponent}
representing a H5 location identifier (file or group). See \code{\link[=H5Fcreate]{H5Fcreate()}},
\code{\link[=H5Fopen]{H5Fopen()}}, \code{\link[=H5Gcreate]{H5Gcreate()}}, \code{\link[=H5Gopen]{H5Gopen()}} to create an object of this kind.}

\item{recursive}{If \code{TRUE}, the content of the whole group hierarchy is
listed. If \code{FALSE}, Only the content of the main group is shown. If a positive
integer is provided this indicates the maximum level of the hierarchy that
is shown.}

\item{load}{If \code{TRUE} the datasets are read in, not only the header
information. Note, that this can cause memory problems for very large files.
In this case choose \code{load=FALSE} and load the datasets successively.}

\item{all}{If \code{TRUE}, a longer list of information on each entry is provided.}

\item{index_type}{See \code{h5const("H5_INDEX")} for possible arguments.}

\item{order}{See \code{h5const("H5_ITER")} for possible arguments.}

\item{s3}{Logical value indicating whether the file argument should be
treated as a URL to an Amazon S3 bucket, rather than a local file path.}

\item{s3credentials}{A list of length three, providing the credentials for
accessing files in a private Amazon S3 bucket.}

\item{\dots}{Arguments passed to \code{\link[=h5read]{h5read()}}}

\item{native}{An object of class \code{logical}. If TRUE, array-like objects
are treated as stored in HDF5 row-major rather than R column-major
orientation. Using \code{native = TRUE} increases HDF5 file portability
between programming languages. A file written with \code{native = TRUE}
should also be read with \code{native = TRUE}}
}
\value{
Returns a hierarchical list structure representing the HDF5
group hierarchy. It either returns the datasets within the list structure
(\code{load=TRUE}) or it returns a \code{data.frame} for each dataset with the
dataset header information (\code{load=FALSE}).
}
\description{
Dump the content of an HDF5 file.
}
\examples{

h5createFile("ex_ls_dump.h5")

# create groups
h5createGroup("ex_ls_dump.h5","foo")
h5createGroup("ex_ls_dump.h5","foo/foobaa")

# write a matrix
B = array(seq(0.1,2.0,by=0.1),dim=c(5,2,2))
attr(B, "scale") <- "liter"
h5write(B, "ex_ls_dump.h5","foo/B")

# list content of hdf5 file
h5dump("ex_ls_dump.h5")

# list content of an hdf5 file in a public S3 bucket
\donttest{
h5dump(file = "https://rhdf5-public.s3.eu-central-1.amazonaws.com/h5ex_t_array.h5", s3 = TRUE)
}

}
\seealso{
\code{\link[=h5ls]{h5ls()}}
}
\author{
Bernd Fischer, Mike L. Smith
}
\keyword{IO}
\keyword{file}
\keyword{interface}
\keyword{programming}
