% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5S.R
\name{H5Sselect_none}
\alias{H5Sselect_none}
\title{Set the selection region of a dataspace to include no elements}
\usage{
H5Sselect_none(h5space)
}
\arguments{
\item{h5space}{\linkS4class{H5IdComponent} object representing a dataspace.}
}
\description{
Set the selection region of a dataspace to include no elements
}
