% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5P.R
\name{H5P_libver_bounds}
\alias{H5P_libver_bounds}
\alias{H5Pset_libver_bounds}
\alias{H5Pget_libver_bounds}
\title{Control the range of HDF5 library versions that will be compatible with
a file.}
\usage{
H5Pset_libver_bounds(
  h5plist,
  libver_low = "H5F_LIBVER_EARLIEST",
  libver_high = "H5F_LIBVER_LATEST"
)

H5Pget_libver_bounds(h5plist)
}
\arguments{
\item{h5plist}{\linkS4class{H5IdComponent} object representing a file access
property list.}

\item{libver_low, libver_high}{Define the earliest and latest versions of
the HDF5 library that will be used when writing object in the file.}
}
\description{
Control the range of HDF5 library versions that will be compatible with
a file.
}
