% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/h5delete.R
\name{h5_deleteAttribute}
\alias{h5_deleteAttribute}
\alias{h5deleteAttribute}
\title{Delete attribute}
\usage{
h5deleteAttribute(file, name, attribute)
}
\arguments{
\item{file}{The filename (character) of the file in which the object is
located.}

\item{name}{The name of the object to which the attribute belongs.}

\item{attribute}{Name of the attribute to be deleted.}
}
\description{
Deletes an attribute associated with a group or dataset within an HDF5 file.
}
\author{
Mike Smith
}
