% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/h5filters.R
\name{h5checkFilters}
\alias{h5checkFilters}
\title{Identifies the filters required to read a dataset
If filters aren't available it will try to identify them
and print the names to the user.}
\usage{
h5checkFilters(h5id)
}
\description{
Identifies the filters required to read a dataset
If filters aren't available it will try to identify them
and print the names to the user.
}
\keyword{internal}
