% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5P.R
\name{H5Pset_istore_k}
\alias{H5Pset_istore_k}
\alias{H5Pget_istore_k}
\title{Get and set the 1/2 rank of an indexed storage B-tree}
\usage{
H5Pset_istore_k(h5plist, ik)

H5Pget_istore_k(h5plist)
}
\arguments{
\item{h5plist}{\linkS4class{H5IdComponent} object representing the file creation property list}

\item{ik}{chunked Storage B-tree 1/2 rank}
}
\description{
Get and set the 1/2 rank of an indexed storage B-tree
}
