% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5L.R
\name{H5Lexists}
\alias{H5Lexists}
\title{Confirm existence of a link}
\usage{
H5Lexists(h5loc, name)
}
\arguments{
\item{h5loc}{An object of class \linkS4class{H5IdComponent} representing a H5
location identifier (file or group).}

\item{name}{The name of the link to be checked}
}
\description{
Confirm existence of a link
}
