% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5F.R
\name{H5Fis_hdf5}
\alias{H5Fis_hdf5}
\title{Determine whether a file is in the HDF5 format}
\usage{
H5Fis_hdf5(name, showWarnings = TRUE)
}
\arguments{
\item{name}{Character vector of length 1, giving the path to the file to
be checked.}

\item{showWarnings}{If the file doesn't exist an warning is generated.
Setting this argument to \code{FALSE} will suppress the warning.}
}
\value{
Returns \code{TRUE}, if the file is an HDF5 file, or \code{FALSE} otherwise.
In the case the file doesn't exist, \code{NA} is returned
}
\description{
\code{H5Fis_hdf5()} determines whether a file is in the HDF5 format.
}
