% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5R.R
\name{H5Rget_obj_type}
\alias{H5Rget_obj_type}
\title{Identify the type of object that a reference points to}
\usage{
H5Rget_obj_type(ref, h5loc)
}
\arguments{
\item{ref}{\code{H5ref} object containing the reference to be queried.}

\item{h5loc}{An \code{H5IdComponent} object representing the file containing the
referenced object.}
}
\value{
Character string of length 1 identifying the object type.  Valid return
values are: \code{"GROUP"}, \code{"DATASET"}, and \code{"NAMED_DATATYPE"}.
}
\description{
Identify the type of object that a reference points to
}
