% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/H5A.R
\name{H5Aclose}
\alias{H5Aclose}
\title{Close an HDF5 attribute}
\usage{
H5Aclose(h5attribute)
}
\arguments{
\item{h5attribute}{An object of class \linkS4class{H5IdComponent} representing a
the attribute to be closed.  Normally created by \code{\link[=H5Aopen]{H5Aopen()}} or similar.}
}
\description{
Close an HDF5 attribute
}
\seealso{
\code{\link[=H5Aopen]{H5Aopen()}}
}
